#!/bin/bash
#############################################################################################
#                                                                                           #
# Aquest script configura els paràmetres de connexió de les unitats compartides del         #
# servidor de centre (Linkat edu)                                                           #
#                                                                                           #
# Llicència: GPL v3.0                                                                       #
# Autor: Joan de Gracia. Servei TAC. Departament d'Ensenyament. Generalitat de Catalunya.   #
# Modificat per Tomàs Sanchez i Pablo Vigo                                                  #
# Versió: 1.0                                                                               #
# Data: 26-07-2013                                                                          #
#############################################################################################


if [ -n "$(id | grep root)" ]; then

{
	zenity --question \
		--text="Aquest script facilita la connexió en Linkat edu \n \
de les unitats de xarxa del servidor de centre. \n\n \
Voleu començar la instal·lació de les unitats de xarxa?" \
		--title="Connexió d'unitats de Xarxa" \
		--ok-label="Sí" \
		--cancel-label="No"
	if [ "$?" != 0 ]; then
	{
		zenity --info --text "S\'avorta la instal·lació" --title='Informació'
		exit 1
	} 
	fi
#
# Es comprova que la distribució base sigui Linkat edu/Ubuntu LTS
#
	LSB=/etc/lsb-release
	if [ -e $LSB ]; then
		if [  "$(grep -i ubuntu $LSB)" = "" ]; then
			zenity --info --text="Sembla que la distribució no sigui Linkat edu.\n\n S\'avorta el programa."
			exit 1
		fi
	fi
#
# Es comprova que l'script no ha estat executat anteriorment.
#
	if [ -e /etc/muntador_unitats ]; then
		zenity --title="Connexió d'unitats de xarxa" --question --text="El configurador ja ha estat aplicat.\n\n Voleu tornar a aplicar-ho?"
		if [ "$?" != 0 ] ; then
			exit 1
		else
			mv /etc/auto.muntador /usr/share/linkat/linkat-muntador-unitats/auto.muntador.$(date +%F)
		fi
	else
		echo "/mnt/Servidor     /etc/auto.muntador --timeout=60 --ghost" >> /etc/auto.master	
		mkdir -p /mnt/Servidor
	fi

#
# Aturem el servei autofs
#
	service autofs stop

#
# RESPOSTA: Aquesta variable s'utilitza per confirmar que la configuració introduïda és correcta.
# NO es realitza cap actuació fins que l'usuari ho confirma.
#
	RESPOSTA=""
	while [ "$RESPOSTA" = "" ]
	do
#
# DIR_TMP: Directori que conté els fitxers de credencials de cadascun dels usuaris del servidor.
# Es tracta d'un directori temporal que s'elimina al final de l'execució de l'script.
#
		DIR_TMP=$(mktemp -d)
#
# FITXER_TMP: Conté la sintaxi per fer el muntatge de les unitats dins del fitxer /etc/auto.muntador.
# Es tracta d'un fitxer temporal que s'elimina al final de l'execució de l'script.
#
		FITXER_TMP=$(mktemp)
#
# FITXER_RECURSOS: Conté el nom dels recursos detectats al servidor de centre.
# Es tracta d'un fitxer temporal que s'elimina al final de l'execució de l'script.
#
		FITXER_RECURSOS=$(mktemp)

		IP=""
		while [ "$IP" = "" ]; do
			IP=$(zenity --title="Introduïu la IP del servidor de centre" --entry --text="Introduïu la IP del servidor del centre" --entry-text="192.168.0.240")
		if [ "$?" = "1" ] ; then
			zenity --error \
			--text="Configuració cancel·lada"
			exit 1
		fi

		done


		RECURSOS="$(smbclient -L $IP -g -N 2>/dev/null|grep -i ^disk|awk -F '|' '{print $2}')"

#
# Comprovem si el servidor necessita usuari i contrasenya i mostrem les unitats compartides
#


zenity --title="Sistema operatiu del Servidor de centre" --question --text="Els Windows Server 2003 i Windows Server 2010 necessiten un usuari i una contrasenya per mostrar les unitats compartides.\n\nSi el vostre servidor és una Linkat, Linux o una altra versió de Windows, no cal que introduïu cap usuari.\n\n Voleu introduir un usuari i contrasenya per veure les unitats compartides?"

		if [ $? -gt 0 ] ; then

			ret="$(smbclient -g -N -L $IP 2> /dev/null | \
			grep ^Disk | \
			perl -ne 'chomp; @ary = split(m/\|/, $_); print "TRUE\n$ary[1]\n$ary[2]\n";' | \
			zenity --list --width 400 --height 300  --title="Unitats compartides al servidor $IP" \
				--text="Seleccioneu quines unitats voleu muntar" \
				--checklist \
				--column="Muntar-la?" \
				--column="Nom de la unitat" \
				--column="Descripció" \
				--separator=" "
			)"
			if [ "$?" = 1 ] ; then
				zenity --error \
				--text="Configuració cancel·lada"
				exit 1
			fi

		else
			WIN=$(zenity --username --password --title="Usuari i contrasenya per al servidor $IP")

                        case $? in
                         0)
                                USUARIWIN=$(echo $WIN | cut -d'|' -f1)
                                PASSWDWIN=$(echo $WIN | cut -d'|' -f2)
                                ;;
                         1)
                                echo "S'ha produït un error."
                                exit 1;;

                        *)
                                exit 1;;
                        esac
			ret="$(smbclient -g -N -L $IP -U $USUARIWIN%$PASSWDWIN 2> /dev/null | \
			grep ^Disk | \
                        perl -ne 'chomp; @ary = split(m/\|/, $_); print "TRUE\n$ary[1]\n$ary[2]\n";' | \
                        zenity --list --width 400 --height 300  --title="Unitats compartides al servidor $IP" \
                                --text="Seleccioneu quines unitats voleu muntar" \
                                --checklist \
                                --column="Muntar-la?" \
                                --column="Nom de la unitat" \
                                --column="Descripció" \
                                --separator=" "
                        )"
                        if [ "$?" = 1 ] ; then
                                zenity --error \
                                --text="Configuració cancel·lada"
                                exit 1
                        fi


		fi

#
# Format HTML al fitxer $FITXER_RECURSOS 
#
echo "<html>" >> $FITXER_RECURSOS
echo "<body>" >> $FITXER_RECURSOS
echo "<table style=\"text-align: left; width: 100%; border=\"0\">" >> $FITXER_RECURSOS
echo "<tr><th>Unitat de xarxa<br></th><th>Usuari<br></th></tr>" >> $FITXER_RECURSOS

zenity --width=400 --info --title="Crear usuaris a l'ordinador" --text="Els usuaris que s'introduexin tot seguit es crearan en aquest ordinador.\n\nAquests usuaris han d'existir al servidor $IP per realitzar la connexió.\n\nAfegiu l'usuari i la contrasenya per connectar amb cadascuna de les diferents unitats de xarxa del servidor $IP.\n\nPer exemple:\n\nRecurs: professorat\nUsuari: prof\n\nRecurs: treball\nUsuari: argo"


		for share in $(echo $ret);
		do
			USUARI=""
			while [ "$USUARI" = "" ]
			do
			
			ENTRY=`zenity --width=400 --username --password --title="Usuari per a la unitat $share"`
			case $? in
		         0)
                		USUARI=$(echo $ENTRY | cut -d'|' -f1)
		                CONTRASENYA=$(echo $ENTRY | cut -d'|' -f2)
                		;;
		         1)
				echo "S'ha produït un error."
                		exit 1;;

		        *)
				exit 1;;
			esac
			done


			RECURS1=$(echo $share|tr '[:lower:]' '[:upper:]')
			case $RECURS1 in
			S)
				PERMISOS_FITXERS="0664"
				PERMISOS_DIRECTORIS="0775"
			;;
			PROGRAMARI)
				PERMISOS_FITXERS="0664"
				PERMISOS_DIRECTORIS="0775"
			;;
			T)
				PERMISOS_FITXERS="0666"
				PERMISOS_DIRECTORIS="0777"
			;;
			TREBALL)
				PERMISOS_FITXERS="0666"
				PERMISOS_DIRECTORIS="0777"
			;;
			*)
				PERMISOS_FITXERS="0660"
				PERMISOS_DIRECTORIS="0770"
			;;
			esac
			printf "$share -fstype=cifs,credentials=/etc/samba/credentials_$USUARI,uid=$USUARI,gid=$USUARI,iocharset=utf8,file_mode=$PERMISOS_FITXERS,dir_mode=$PERMISOS_DIRECTORIS ://$IP/$share\n" >> $FITXER_TMP 	
			printf "username=$USUARI\npassword"="$CONTRASENYA\n" > $DIR_TMP/credentials_$USUARI
			echo "<tr><td>$share</td><td>$USUARI</td></tr>" >> $FITXER_RECURSOS
		done
		

#
# Format HTML al fitxer $FITXER_RECURSOS
#
echo "</table>" >> $FITXER_RECURSOS
echo "</body>" >> $FITXER_RECURSOS
echo "</html>" >> $FITXER_RECURSOS


		zenity --text-info \
			--filename=$FITXER_RECURSOS \
			--title="Confirmació de la configuració" \
			--html 
		if [ "$?" = 1 ] ; then
			zenity --error \
			--text="Configuració cancel·lada"
			RESPOSTA=""
#
# Eliminació dels fitxers/directoris temporals
# En cas de cancel·lació de l'aplicatiu per part de l'usuari, els fitxers/directoris temporals
# s'eliminen perquè no interfereixi la informació errònia amb la correcta.
#
			rm $FITXER_TMP $FITXER_RECURSOS
			rm -rf $DIR_TMP
		else
			RESPOSTA="1"
			cp $FITXER_TMP /etc/auto.muntador
		fi
	done

#
# Canvi permisos dels fitxers credentials
#
	cp $DIR_TMP/credentials* /etc/samba/
	chmod 640 /etc/samba/credentials*
	chown root:root /etc/samba/credentials*
#
# Creació d'usuaris. S'obtenen dels fitxers "credentials"
# S'accepta que els usuaris puguin tenir contrasenya en blanc
# Els usuaris es creen al directori /home-local
#
	USUARI=""
	CONTRASENYA=""
	HOME_LOCAL="/home-local"
	if [ ! -d $HOME_LOCAL ]; then
		mkdir -p $HOME_LOCAL
	fi
	for i in $(ls $DIR_TMP | grep -i credentials)
	do
		USUARI=$(grep -i username $DIR_TMP/$i | awk -F '=' '{print $2}')
		CONTRASENYA=$(grep -i password $DIR_TMP/$i | awk -F '=' '{print $2}')
		if [ "$CONTRASENYA" = "" ]; then
			CONTRASENYA_XIFRADA=""
		else
			CONTRASENYA_XIFRADA="$(mkpasswd $CONTRASENYA)"
		fi

# Es comprova que l'usuari no existeixi ja al sistema
#
		if [ "$(id $USUARI 2>/dev/null)" = "" ];
		then
			useradd -c "Usuari $USUARI" -s /bin/bash -d $HOME_LOCAL/$USUARI -m -p "$CONTRASENYA_XIFRADA" $USUARI
		fi

		ln -s /mnt/Servidor /home/$USUARI/Escriptori/Unitats\ xarxa > /dev/null 2>&1
		ln -s /mnt/Servidor /home-local/$USUARI/Escriptori/Unitats\ xarxa > /dev/null 2>&1
	done

#
# Es crea el fitxer muntador_unitats que serveix per detectar si l'script ha estat executat o no.
# Es munten les unitats del servidor en acabar l'execució de l'script.
#
	touch /etc/muntador_unitats
	service autofs start
#
# Eliminació dels fitxers/directoris temporals
#
	rm $FITXER_TMP $FITXER_RECURSOS
	rm -rf $DIR_TMP
	zenity --info --text "Procés finalitzat amb èxit." --title="Conexio d'unitats de Xarxa"
}
else
	zenity --info --text "No sou usuari root i no podeu executar el programa" --title="Conexio d'unitats de xarxa"
	exit 1
fi
